from typing import Dict, Optional, List, Union
import yaml

from opa_tools.config import REQUEST_DIR


def _get_single_request(num: int, kind: str) -> Dict:
    if kind not in ("opa", "gsv"):
        raise AssertionError(
            f"kind must be either 'opa' or 'gsv', but was '{kind}'."
        )
    request_path = REQUEST_DIR / f"{num:02d}" / f"{kind}.yml"
    with open(request_path, "r", encoding="utf-8") as rf:
        return yaml.safe_load(rf)

def get_request(num: Optional[int] = None, /, kind: Optional[str] = None, *, single_as_list=False) -> Union[List[Dict], Dict]:
    if num is None:
        output = []
        for num in get_request_names():
            output.extend(get_request(num, kind=kind, single_as_list=True))
        return output
    if kind is None:
        return [
            _get_single_request(num=num, kind="gsv"),
            _get_single_request(num=num, kind="opa"),
        ]
    if single_as_list:
        return [_get_single_request(num=num, kind=kind)]
    return _get_single_request(num=num, kind=kind)


def get_request_names():
    return sorted(int(path.name) for path in REQUEST_DIR.iterdir())

